import React from 'react'

import {
  useDisclosure,
  AlertDialog,
  AlertDialogOverlay,
  AlertDialogContent,
  AlertDialogHeader,
  AlertDialogBody,
  AlertDialogFooter,
  Button,
  Text,
  Tooltip,
  ButtonProps,
  useToast,
} from '@chakra-ui/react'
import { useQueryClient } from 'react-query'

import { Pattern } from 'src/models'

import { DeletePatternMutation, GetPatternsQuery } from '../query-builder'
import { useLocation } from 'react-use'
import { navigate, routes } from '@redwoodjs/router'
import { useTranslation } from 'react-i18next'
import { capitalize } from 'lodash'

export type DeletePatternButtonProps = {
  pattern: Pattern
  hasTooltip?: boolean
  children: React.ReactElement<ButtonProps>
}

export const DeletePatternButton: React.FC<DeletePatternButtonProps> = ({
  pattern,
  hasTooltip = false,
  children,
}) => {
  const { t } = useTranslation()
  const toast = useToast()
  const queryClient = useQueryClient()
  const [isLoading, setIsLoading] = React.useState(false)
  const { isOpen, onOpen, onClose } = useDisclosure()
  const cancelRef = React.useRef<HTMLButtonElement>(null)
  const { pathname } = useLocation()

  const deletePatternMutation = DeletePatternMutation.useMutation({
    onSuccess: () => {
      setIsLoading(false)
      toast({
        status: 'success',
        position: 'top',
        title: capitalize(t('patterns_delete_pattern_success')),
      })
      queryClient.invalidateQueries(GetPatternsQuery.getKey({}))
      if (pathname.startsWith('/patterns/')) {
        // @ts-ignore -- TODO
        navigate(routes.patterns())
      }
    },
    onError: () => {
      toast({
        status: 'error',
        position: 'top',
        title: capitalize(t('patterns_delete_pattern_success')),
      })
      onClose()
      setIsLoading(false)
    },
  })

  const handleOnDelete = () => {
    setIsLoading(true)
    deletePatternMutation.mutate({ id: pattern.id })
  }

  if (!React.isValidElement(children) || Array.isArray(children)) {
    throw new Error(
      'DeletePatternButton child must be a single clickable element'
    )
  }

  const btn = React.cloneElement(children, { onClick: onOpen })

  return (
    <>
      <Tooltip
        isDisabled={!hasTooltip}
        label={capitalize(
          t('patterns_delete_pattern_confirmation_modal_header')
        )}
      >
        {btn}
      </Tooltip>
      <AlertDialog
        isOpen={isOpen}
        onClose={onClose}
        leastDestructiveRef={cancelRef}
      >
        <AlertDialogOverlay>
          <AlertDialogContent>
            <AlertDialogHeader fontSize="lg" fontWeight="bold">
              {capitalize(
                t('patterns_delete_pattern_confirmation_modal_header')
              )}{' '}
              {pattern.name}
            </AlertDialogHeader>
            <AlertDialogBody>
              <Text>
                {capitalize(
                  t('patterns_delete_pattern_confirmation_modal_body')
                )}
              </Text>
            </AlertDialogBody>
            <AlertDialogFooter>
              <Button ref={cancelRef} variant="ghost" onClick={onClose}>
                {capitalize(t('cancel'))}
              </Button>
              <Button
                variant="solid"
                onClick={handleOnDelete}
                ml="3"
                isLoading={isLoading}
              >
                {capitalize(t('delete'))}
              </Button>
            </AlertDialogFooter>
          </AlertDialogContent>
        </AlertDialogOverlay>
      </AlertDialog>
    </>
  )
}
