import React from 'react'

import {
  VStack,
  TableContainer,
  Table,
  Thead,
  Tr,
  Th,
  Tbody,
  Td,
  Text,
  TableCaption,
  Tooltip,
  HStack,
  Icon,
  IconButton,
  Spinner,
  useDisclosure,
} from '@chakra-ui/react'
import { FaEdit, FaEye, FaPlus } from 'react-icons/fa'
import { match } from 'ts-pattern'

import { Link, routes } from '@redwoodjs/router'

import { Pattern } from 'src/models'

import { AddPatternButton } from './add-pattern'
import { GetPatternsQuery } from './query-builder'
import { FaTrash } from 'react-icons/fa'
import { EditPatternModal } from './edit-pattern'
import { AddImplementationModal } from '../implementations/add-implementation'
import { DeletePatternButton } from './delete-pattern'
import { useTranslation } from 'react-i18next'
import { capitalize } from 'lodash'

export const Patterns: React.FC = () => {
  const patternsQuery = GetPatternsQuery.useQuery({})
  const { t } = useTranslation()

  const tableHeaders = [capitalize(t('name')), capitalize(t('description')), '']

  return (
    <VStack alignItems="flex-start" spacing="2">
      <Text fontWeight="bold" fontSize="x-large" px="1">
        {capitalize(t('patterns_patterns'))}
      </Text>
      <TableContainer>
        <Table variant="striped" size="lg" minW="80vw">
          <TableCaption>
            {capitalize(t('patterns_available_pattarns'))}
          </TableCaption>
          <Thead>
            <Tr>
              {tableHeaders.map((header, index) => (
                <Th color="gray.500" key={`th-${index}`}>
                  {header}
                </Th>
              ))}
            </Tr>
          </Thead>
          <Tbody>
            {match(patternsQuery)
              .with({ status: 'success', data: [] }, () => (
                <Text>{capitalize(t('patterns_not_found'))}</Text>
              ))
              .with({ status: 'success' }, ({ data }) =>
                data.map((pattern, index) => (
                  <Tr key={`pattern-${pattern.name}-${index}`}>
                    <Td>
                      {/* @ts-ignore -- TODO */}
                      <Link to={routes.pattern({ id: pattern.id })}>
                        {pattern.name}
                      </Link>
                    </Td>
                    <Td>
                      <Tooltip label={pattern.description} hasArrow>
                        <Text isTruncated maxW="sm">
                          {pattern.description}
                        </Text>
                      </Tooltip>
                    </Td>

                    <Td>
                      <ActionsRow pattern={pattern} />
                    </Td>
                  </Tr>
                ))
              )
              .with({ status: 'loading' }, () => (
                <Tr>
                  <Td colSpan={tableHeaders.length}>
                    <HStack justifyContent="center">
                      <Spinner />
                    </HStack>
                  </Td>
                </Tr>
              ))
              .otherwise(() => null)}
          </Tbody>
        </Table>
      </TableContainer>
      <HStack w="full" justifyContent="flex-end">
        <AddPatternButton />
      </HStack>
    </VStack>
  )
}

type ActionsRowProps = {
  pattern: Pattern
}
const ActionsRow: React.FC<ActionsRowProps> = ({ pattern }) => {
  const { t } = useTranslation()
  const editPatternDisclosure = useDisclosure()
  const addImplementationDisclosure = useDisclosure()

  return (
    <>
      <HStack spacing="0">
        {/* @ts-ignore -- TODO */}
        <Link to={routes.pattern({ id: pattern.id })}>
          <Tooltip hasArrow shouldWrapChildren label={capitalize(t('view'))}>
            <IconButton
              aria-label="view pattern"
              variant="unstyled"
              icon={<Icon as={FaEye} />}
            />
          </Tooltip>
        </Link>

        <Tooltip
          hasArrow
          shouldWrapChildren
          label={capitalize(t('implementations_add_implementation'))}
        >
          <IconButton
            aria-label="add implementation"
            variant="unstyled"
            onClick={addImplementationDisclosure.onOpen}
            icon={<Icon as={FaPlus} />}
          />
        </Tooltip>

        <Tooltip hasArrow shouldWrapChildren label={capitalize(t('edit'))}>
          <IconButton
            aria-label="edit pattern"
            variant="unstyled"
            onClick={editPatternDisclosure.onOpen}
            icon={<Icon as={FaEdit} />}
          />
        </Tooltip>
        <DeletePatternButton pattern={pattern} hasTooltip={true}>
          <IconButton
            aria-label="remove pattern"
            variant="unstyled"
            icon={<Icon as={FaTrash} />}
          />
        </DeletePatternButton>
      </HStack>
      {editPatternDisclosure.isOpen && (
        <EditPatternModal
          disclosure={editPatternDisclosure}
          pattern={pattern}
        />
      )}
      {addImplementationDisclosure.isOpen && (
        <AddImplementationModal
          disclosure={addImplementationDisclosure}
          pattern={pattern}
        />
      )}
    </>
  )
}
